#include <iostream>
#include <fstream>
#include <string>

#include <string.h>
#include "config.h" // for HAVE_LIBTHREAD definitions
#include "support/searchpath.h" // for platform-specific path delimiters.
#include "argyll_support.h"

#include "support/simplecombo.h" // For InstrumentSelector widget

using namespace std;

#include "gettext.h"
#define _(x) gettext(x)
#define N_(x) gettext_noop(x)


// Static support function

const char *ArgyllSupport::GetColorantLongName(const char *shortname)
{
	const char *cn=shortname;
	if(*cn)
	{
		cerr << "Fetching longname for: " << cn << endl;
		if(cn[0]=='2')
		{
			switch(cn[1])
			{
				case 'c':
					return("Medium Cyan");
					break;
				case 'm':
					return("Medium Magenta");
					break;
				case 'y':
					return("Medium Yellow");
					break;
				case 'k':
					return("Medium Black");
					break;
			}
		}
		if(cn[0]=='1')
		{
			switch(cn[1])
			{
				case 'k':
					return("Light Light Black");
					break;
			}
		}
		else
		{
			switch(cn[0])
			{
				case 'C':
					return("Cyan");
					break;
				case 'M':
					return("Magenta");
					break;
				case 'Y':
					return("Yellow");
					break;
				case 'K':
					return("Black");
					break;
				case 'c':
					return("Light Cyan");
					break;
				case 'm':
					return("Light Magenta");
					break;
				case 'y':
					return("Light Yellow");
					break;
				case 'k':
					return("Light Black");
					break;
				case 'R':
					return("Red");
					break;
				case 'B':
					return("Blue");
					break;
				case 'G':
					return("Green");
					break;
				case 'O':
					return("Orange");
					break;
			}
		}
	}
	return(NULL);
}


ArgyllSupport::ArgyllSupport(const char *binpath) : SearchPathHandler()
{
	AddPath(binpath);
}


ArgyllSupport::~ArgyllSupport()
{
}


bool ArgyllSupport::CheckBinPath()
{
#ifdef WIN32
	const char *utils[]=
	{
		"tiffgamut.exe",
		"targen.exe",
		"printtarg.exe",
		"collink.exe",
		"colprof.exe"
	};
#else
	const char *utils[]=
	{
		"tiffgamut",
		"targen",
		"printtarg",
		"collink",
		"colprof"
	};
#endif
	for(unsigned int i=0;i<(sizeof(utils)/sizeof(const char *));++i)
	{
		char *cmd=SearchPaths(utils[i]);
		if(cmd)
			free(cmd);
		else
			return(false);
	}
	return(true);
}


bool ArgyllSupport::CallArgyll(const char *command,Progress *prog)
{
	// Here we use a subthread and system calls to run an Argyll command in the background.
	// In the foreground we run a progress loop to keep a progress meter updated.
	// If no progress meter is supplied, we wait rather than poll.
	if(prog)
		prog->DoProgress(0,0);
	Thread t(thread_entry,(void *)command);
	t.Start();
	t.WaitSync();
	if(prog)
	{
		while(!t.TestFinished())
		{
			if(prog)
				prog->DoProgress(0,0);
	#ifdef WIN32
			Sleep(50);
	#else
			usleep(50000);
	#endif
		}
	}
	else
		t.WaitFinished();
	int result=t.GetReturnCode();
	return(result==0);
}


int ArgyllSupport::thread_entry(Thread *t,void *ud)
{
	int returncode=0;
	const char *cmd=(const char *)ud;
	t->SendSync();
	try
	{
		if(system(cmd))
			throw "Command failed";
	}
	catch(const char *err)
	{
		cerr << "Subthread error: " << err << endl;
		returncode=-1;
	}
	return(returncode);
}


/////////////////////////////////////////////////////////////////////////////////////////////////////

static struct {const char *name; int width; int height; } papersizes[] =
{
	{"A4",210,297.0},
	{"A4R",297,210.0},
	{"A3",297,420.0},
	{"A2",420,594.0},
	{"Letter",216,279},
	{"LetterR",279,216},
	{"Legal",216,356},
	{"11x17",279,432},
	{NULL,0,0}
};

ArgyllPaperSize::ArgyllPaperSize(const char *name) : size(NULL)
{
	int i=0;
	while(papersizes[i].name)
	{
		if(strcasecmp(papersizes[i].name,name)==0)
		{
			size=papersizes[i].name;
			width=papersizes[i].width;
			height=papersizes[i].height;
			break;
		}
		++i;
	}
	if(!size)
		throw "Unknown papersize";
}


ArgyllPaperSize::ArgyllPaperSize(int w,int h) : size(NULL), width(w), height(h)
{
}


int ArgyllPaperSize::GetWidth()
{
	return(width);
}


int ArgyllPaperSize::GetHeight()
{
	return(height);
}


char *ArgyllPaperSize::GetPrinttargArgs()
{
	char *result=NULL;
	if(size)
	{
		const char *fmt="-p%s";
		result=(char *)malloc(strlen(fmt)+strlen(size));
		sprintf(result,fmt,size);
	}
	else
	{
		const char *fmt="-p%dx%d";
		result=(char *)malloc(strlen(fmt)+20);
		sprintf(result,fmt,width,height);
	}
	return(result);
}


///////////////////////////////////////////////////////////////////////////////////////////////////


static struct instrumentdesc {
	const char *name;
	const char *description;
	const char *args;
} instruments[] =
{
	{"DTP20",N_("X-Rite DTP20"),"-i20"},
	{"DTP22",N_("X-Rite DTP22"),"-i22"},
	{"DTP41",N_("X-Rite DTP41"),"-i41"},
	{"DTP51",N_("X-Rite DTP51"),"-i51"},
	{"SpectroScan",N_("GMB SpectroScan"),"-iSS"},
	{"SpectroScan (Hex patches)",N_("GMB SpectroScan (Hex patches)"),"-iSS -h"},
	{"i1Pro",N_("X-Rite i1Pro"),"-ii1"},
	{"Scanner",N_("Desktop Scanner"),"-iSS -s"},
};


ArgyllInstrument::ArgyllInstrument(const char *name) : inst(0)
{
	while(inst<(sizeof(instruments)/sizeof(instrumentdesc)))
	{
		if(strcasecmp(name,instruments[inst].name)==0)
			return;
		++inst;
	}
	throw "Unknown instrument";
}


ArgyllInstrument::~ArgyllInstrument()
{
}


char *ArgyllInstrument::GetPrinttargArgs()
{
	return(strdup(instruments[inst].args));
}


int ArgyllSupport::GetSupportedInstrumentCount()
{
	return(sizeof(instruments)/sizeof(instrumentdesc));
}


const char *ArgyllSupport::GetInstrumentName(unsigned int inst)
{
	while(inst<(sizeof(instruments)/sizeof(instrumentdesc)))
		return(instruments[inst].name);
	return(NULL);
}


const char *ArgyllSupport::GetInstrumentDescription(unsigned int inst)
{
	while(inst<(sizeof(instruments)/sizeof(instrumentdesc)))
		return(instruments[inst].description);
	return(NULL);
}


