#include <iostream>

#include "config.h" // For HAVE_LIBPTHREADS
#include "argyll_support.h"
#include "channelresponse.h"

using namespace std;

RawChannelResponses::RawChannelResponses(const char *filename)
	: TI3File(filename), DeviceNColorantList(), firstchannel(NULL), paper()
{
	// Deduce paper-white from the data.  This only works for Subtractive colour,
	// since we check for colorants all set to zero.  Since there are commonly several
	// unprinted patches, we average them.

	int papercount=0;
	for(int i=0;i<GetSets();++i)
	{
		double acc=0.0;
		double **data=GetData(i);
		for(int j=0;j<colorants;++j)
		{
			acc+=*data[j];
		}
		if(acc==0.0)
		{
			paper.X+=*data[Xindex];
			paper.Y+=*data[Yindex];
			paper.Z+=*data[Zindex];
			++papercount;
		}
	}
	if(papercount)
	{
		paper.X/=papercount;
		paper.Y/=papercount;
		paper.Z/=papercount;
	}
	cerr << papercount << " unprinted patches found - Paper colour (XYZ): " << paper.X << ", " << paper.Y << ", " << paper.Z << endl;

	// Generate the channel responses - one for each colorant
	for(int i=0;i<colorants;++i)
	{
		new ChannelResponse(this,i);
	}
}


RawChannelResponses::~RawChannelResponses()
{
	while(firstchannel)
		delete firstchannel;
}


ChannelResponse *RawChannelResponses::FirstChannel()
{
	return(firstchannel);
}


ChannelResponseSample *RawChannelResponses::GetPaper()
{
	return(&paper);
}

////////////////////////////////////////////////////////////////////////////////////////////////


ChannelResponse::ChannelResponse(RawChannelResponses *header,int channel)
	: channelindex(channel), channelname(NULL), samplecount(0), samples(NULL), header(header), nextchannel(NULL), prevchannel(NULL)
{
	// We sort out the colorant stuff first because if an unknown colorant is going
	// to trigger an exception in the DeviceNColorant contructor we want it to
	// happen before we're linked into the list.
	channelname=header->GetFieldName(channel);
	cerr << "Created channel for: " << channelname << endl;
	const char *sn=GetShortName();
	const char *longname=ArgyllSupport::GetColorantLongName(sn);
	cerr << "Creating colorant for " << longname << endl;
	new DeviceNColorant(*header,longname);


	if((prevchannel=header->firstchannel))
	{
		while(prevchannel->nextchannel)
			prevchannel=prevchannel->nextchannel;
		prevchannel->nextchannel=this;
	}
	else
		header->firstchannel=this;


	// We count the number of patches which involve just this channel

	for(int i=0;i<header->GetSets();++i)
	{
		double chan=0.0;
		double acc=0.0;
		double **data=header->GetData(i);
		for(int j=0;j<header->colorants;++j)
		{
			if(j==channelindex)
				chan=*data[j];
			else
				acc+=*data[j];
		}
		// Chan contains the contents of the row for this channel
		// acc contains the sum total of the other colorants' values.
		if(acc<0.01 && chan>0.01)
		{
			++samplecount;
		}
	}
	cerr << "Found " << samplecount << "patches in total" << endl;
	samples=new ChannelResponseSample[samplecount+1];
	samples[0]=header->paper;
	cerr << "paper colour assigned: " << header->paper.X << " -> " << samples[0].X << endl;
	cerr << "" << header->paper.Y << " -> " << samples[0].Y << endl;
	cerr << "" << header->paper.Z << " -> " << samples[0].Z << endl;
	ChannelResponseSample *ptr=samples+1;
	for(int i=0;i<header->GetSets();++i)
	{
		double chan=0.0;
		double acc=0.0;
		double **data=header->GetData(i);
		for(int j=0;j<header->colorants;++j)
		{
			if(j==channelindex)
				chan=*data[j];
			else
				acc+=*data[j];
		}
		// Chan contains the contents of the row for this channel
		// acc contains the sum total of the other colorants' values.
		if(acc<0.01 && chan>0.01)
		{
			ptr->input=chan;
			ptr->X=*data[header->Xindex];
			ptr->Y=*data[header->Yindex];
			ptr->Z=*data[header->Zindex];
			++ptr;
		}
	}
}


ChannelResponse::~ChannelResponse()
{
	if(samples)
		delete[] samples;
	cerr << "Removing channel: " << GetShortName() << endl;
	if(prevchannel)
		prevchannel->nextchannel=nextchannel;
	else
		header->firstchannel=nextchannel;
	if(nextchannel)
		nextchannel->prevchannel=prevchannel;
}


ChannelResponse *ChannelResponse::NextChannel()
{
	return(nextchannel);
}


ChannelResponse *ChannelResponse::PrevChannel()
{
	return(prevchannel);
}


const char *ChannelResponse::GetName()
{
	return(channelname);
}


const char *ChannelResponse::GetShortName()
{
	if(channelname)
	{
		int i=0;
		while(channelname[i])
		{
			if(channelname[i]=='_')
			{
				return(channelname+i+1);
			}
			++i;
		}
	}
	return(NULL);
}


int ChannelResponse::GetChannelIndex()
{
	return(channelindex);
}

int ChannelResponse::GetSampleCount()
{
	return(samplecount);
}

ChannelResponseSample *ChannelResponse::GetSamples()
{
	return(samples);
}

