#include <iostream>
#include <stdlib.h>
#include <string.h>

#include "imagesource_util.h"
#include "imagesource_downsample.h"
#include "imagesource_scaledensity.h"
#include "imagesource_gamma.h"
#include "tiffsave.h"
#include "progresstext.h"



class ImageSource_DownRender : public ImageSource
{
	public:
	ImageSource_DownRender(ImageSource *source,int width,int height,float density,float gamma);
	virtual ~ImageSource_DownRender();
	ISDataType *GetRow(int row);
	double *GetVScaled(int row);	// Calls GetHScaled to do its job
	double *GetHScaled(int row);
	private:
	ImageSource *source;
	double *hbuffer;				// Buffer filled in and returned by GetHScaled
	int hrow;
	double *vbuffer;				// Buffer filled in and returned by GetVScaled
	int vrow;
	int srcrow;
	int acc;
	double *tmp;					// Temp buffer used by GetVScaled
	float density;
	float gamma;
};


ImageSource_DownRender::~ImageSource_DownRender()
{
	if(source)
		delete source;
	if(hbuffer)
		free(hbuffer);
	if(vbuffer)
		free(vbuffer);
	if(tmp)
		free(tmp);
}


ISDataType *ImageSource_DownRender::GetRow(int row)
{
	if(row==currentrow)
		return(rowbuffer);

	double *src=GetVScaled(row);

	switch(STRIP_ALPHA(type))
	{
		case IS_TYPE_RGB:	// RGB is the only colourspace in which we treat 0 as black.
			if(HAS_ALPHA(type))	// We don't want to scale the alpha channel!
			{
				for(int i=0;i<width;++i)
				{
					int j;
					for(j=0;j<samplesperpixel-1;++j)
					{
						double t=src[i*samplesperpixel+j];
						t=IS_SAMPLEMAX - (density * (IS_SAMPLEMAX-t));
						t=pow(t/IS_SAMPLEMAX,gamma);
						if(t>1.0) t=1.0;
						rowbuffer[i*samplesperpixel+j]=ISDataType(IS_SAMPLEMAX*t);
					}
					rowbuffer[i*samplesperpixel+j]=ISDataType(src[i*samplesperpixel+j]);	// Copy alpha unchanged
				}
			}
			else
			{
				for(int i=0;i<width;++i)
				{
					for(int j=0;j<samplesperpixel;++j)
					{
						double t=src[i*samplesperpixel+j];
						t=IS_SAMPLEMAX - (density * (IS_SAMPLEMAX-t));
						t=pow(t/IS_SAMPLEMAX,gamma);
						if(t>1.0) t=1.0;
						rowbuffer[i*samplesperpixel+j]=ISDataType(IS_SAMPLEMAX*t);
					}
				}	
			}		
			break;
		default:
			if(HAS_ALPHA(type))	// We don't want to scale the alpha channel!
			{
				for(int i=0;i<width;++i)
				{
					int j;
					for(j=0;j<samplesperpixel-1;++j)
					{
						int t=int(density * src[samplesperpixel+j]);
						t=1.0-pow(1.0-(t/IS_SAMPLEMAX),gamma);
						if(t>1.0) t=1.0;
						rowbuffer[i*samplesperpixel+j]=IS_SAMPLEMAX*t;
					}
					rowbuffer[i*samplesperpixel+j]=src[i*samplesperpixel+j];	// Copy alpha unchanged
				}	
			}
			else
			{
				for(int i=0;i<width;++i)
				{
					for(int j=0;j<samplesperpixel;++j)
					{
						int t=int(density * src[samplesperpixel+j]);
						t=1.0-pow(1.0-(t/IS_SAMPLEMAX),gamma);
						if(t>1.0) t=1.0;
						rowbuffer[i*samplesperpixel+j]=IS_SAMPLEMAX*t;
					}
				}	
			}		
			break;
	}

	currentrow=row;
	return(rowbuffer);
}


double *ImageSource_DownRender::GetVScaled(int row)
{
	if(vrow==row)
		return(vbuffer);
	vrow=row;
	double *srcdata;

	// Add the smaller value (destination width)
	acc+=height;

	// As long as the counter is less than the larger value, we take full pixels.
	while(acc<source->height)
	{
		if(srcrow>=source->height)
			srcrow=source->height-1;
		srcdata=GetHScaled(srcrow++);
		for(int i=0;i<width*samplesperpixel;++i)
			tmp[i]+=srcdata[i];
		acc+=height;
	}

	double p=source->height-(acc-height);
	p/=height;
	// p now contains the proportion of the next row to be counted towards the output row.

	acc-=source->height;
	// And acc now contains the remainder, ready for the next round.

	// So we add p * the new source pixel to the current output pixel...
	if(srcrow>=source->height)
		srcrow=source->height-1;
	srcdata=GetHScaled(srcrow);
	for(int i=0;i<width*samplesperpixel;++i)
		tmp[i]+=p*srcdata[i];

	// Store it...
	for(int i=0;i<width*samplesperpixel;++i)
		vbuffer[i]=(tmp[i]*height)/source->height;

	// And start off the next output pixel with (1-p) * the source pixel.
	for(int i=0;i<width*samplesperpixel;++i)
		tmp[i]=(1.0-p)*srcdata[i];
	++srcrow;

	return(vbuffer);
}


double *ImageSource_DownRender::GetHScaled(int row)
{
	if(hrow==row)
		return(hbuffer);
	hrow=row;

	// We accumulate pixel values from a potentially
	// large number of pixels and process all the samples
	// in a pixel at one time.
	double tmp[IS_MAX_SAMPLESPERPIXEL];
	for(int i=0;i<samplesperpixel;++i)
		tmp[i]=0;

	ISDataType *srcdata=source->GetRow(row);

	// We use a Bresenham-esque method of calculating the
	// pixel boundaries for scaling - add the smaller value
	// to an accumulator until it exceeds the larger value,
	// then subtract the larger value, leaving the remainder
	// in place for the next round.
	int a=0;
	int src=0;
	int dst=0;
	while(dst<width)
	{
		// Add the smaller value (destination width)
		a+=width;

		// As long as the counter is less than the larger value
		// (source width), we take full pixels.
		while(a<source->width)
		{
			if(src>=source->width)
				src=source->width-1;
			for(int i=0;i<samplesperpixel;++i)
				tmp[i]+=srcdata[samplesperpixel*src+i];
			++src;
			a+=width;
		}

		double p=source->width-(a-width);
		p/=width;
		// p now contains the proportion of the next pixel
		// to be counted towards the output pixel.

		a-=source->width;
		// And a now contains the remainder,
		// ready for the next round.

		// So we add p * the new source pixel
		// to the current output pixel...
		if(src>=source->width)
			src=source->width-1;
		for(int i=0;i<samplesperpixel;++i)
			tmp[i]+=p*srcdata[samplesperpixel*src+i];

		// Store it...
		for(int i=0;i<samplesperpixel;++i)
		{
			hbuffer[samplesperpixel*dst+i] = (tmp[i]*width)/source->width;
		}
		++dst;

		// And start off the next output pixel with
		// (1-p) * the source pixel.
		for(int i=0;i<samplesperpixel;++i)
			tmp[i]=(1.0-p)*srcdata[samplesperpixel*src+i];
		++src;
	}

	return(hbuffer);
}


ImageSource_DownRender::ImageSource_DownRender(ImageSource *source,int width,int height,float density,float gamma)
	: ImageSource(source), source(source), hbuffer(NULL), vbuffer(NULL), srcrow(0), acc(0), tmp(NULL), density(density), gamma(gamma)
{
	cerr << "Using Downrender filter" << endl;
	this->width=width;
	this->height=height;
	yres=(source->yres*height); yres/=source->height;
	xres=(source->xres*width); xres/=source->width;
	randomaccess=false;

	// Make rendering buffer for vdownsample at new width
	if(!(tmp=(double *)malloc(sizeof(double)*width*samplesperpixel)))
		throw "Can't allocate scaling buffer!";
	for(int i=0;i<width*samplesperpixel;++i)
		tmp[i]=0.0;

	// Make rendering buffer for vdownsample at new width
	if(!(vbuffer=(double *)malloc(sizeof(double)*width*samplesperpixel)))
		throw "Can't allocate scaling buffer!";
	for(int i=0;i<width*samplesperpixel;++i)
		vbuffer[i]=0.0;

	// Make rendering buffer for hdownsample at new width
	if(!(hbuffer=(double *)malloc(sizeof(double)*width*samplesperpixel)))
		throw "Can't allocate scaling buffer!";
	for(int i=0;i<width*samplesperpixel;++i)
		hbuffer[i]=0.0;
	hrow=-1;
	vrow=-1;
	MakeRowBuffer();
}


int main(int argc,char **argv)
{
	try
	{
		char *infile=argv[1];
		int percent=100;
		float density=1.0;
		float gamma=1.0;
		const char *outfile="";
		switch(argc)
		{
			case 3:
				outfile=argv[2];
				break;
			case 4:
				percent=atoi(argv[2]);
				outfile=argv[3];
				break;
			case 6:
				percent=atoi(argv[2]);
				density=atof(argv[3]);
				gamma=atof(argv[4]);
				outfile=argv[5];
				break;
			default:
				cerr << "Downrender - for processing the output of Gutenprint's 'unprint' utility" << endl;
				cerr << "Usage: " << argv[0] << " in.pnm [percent] [density gamma] out.tif" << endl;
				break;
		}
		ImageSource *is=ISLoadImage(argv[1]);
//		is=new ImageSource_Downsample(is,(is->width*percent)/100,(is->height*percent)/100);
//		is=new ImageSource_DownRender(is,1.0/density);
//		ImageSource_Gamma dst(is,1.0/gamma);
		ImageSource_DownRender dst(is,(is->width*percent)/100,(is->height*percent)/100,1.0/density,1.0/gamma);
		TIFFSaver ts(outfile,&dst);
		ProgressText p;
		ts.SetProgress(&p);
		ts.Save();
	}
	catch(const char *err)
	{
		cerr << "Error: " << err << endl;
	}
	return(0);
}
