#include <iostream>
#include <string.h>

#include <gutenprint/gutenprint.h>

#include "gpdevicensupport.h"

using namespace std;


int GetPrinterColorantCount(stp_vars_t *vars)
{
	int result=0;
	stp_parameter_t desc;
	stp_describe_parameter(vars,"ChannelNames",&desc);
	stp_string_list_t *strlist=desc.bounds.str;
	if(strlist)
		result=stp_string_list_count(strlist);
	stp_parameter_description_destroy(&desc);
	return(result);
}


//////////////////////////////////////////////////////////////////////////////


GPColorantList::GPColorantList(stp_vars_t *vars) : DeviceNColorantList()
{
	BuildList(vars);
}


GPColorantList::GPColorantList(GPrinterSettings *gp) : DeviceNColorantList()
{
	BuildList(gp->stpvars);
}


GPColorantList::~GPColorantList()
{
}


GPColorant *GPColorantList::First()
{
	return((GPColorant *)FirstColorant());
}


// Create a string representation of which colorants are active,
// suitable for storing in a config file.

char *GPColorantList::GetEnabledColorants()
{
	char *result=NULL;
	GPColorant *col=First();
	int len=2;  // Initial ":" and null-terminator
	while(col)
	{
		if(col->GetEnabled())
		{
			const char *name=col->GetName();
			if(name)
				len+=strlen(name)+2;
		}
		col=col->NextColorant();
	}
	result=(char *)malloc(len);
	result[0]=':';
	result[1]=0;
	col=First();
	while(col)
	{
		if(col->GetEnabled())
		{
			const char *name=col->GetName();
			if(name)
			{
				strcat(result,name);
				strcat(result,":");
			}
		}
		col=col->NextColorant();
	}
	if(!result)
		result=strdup("");
	return(result);
}


// Given a string respresention as produced by GetEnabledColorants()
// sets the "enabled" flag in the colorants

void GPColorantList::SetEnabledColorants(const char *colstr)
{
	if(colstr && strlen(colstr))
	{
		GPColorant *col=First();
		int len=1;
		while(col)
		{
			col->Disable();
			const char *name=col->GetName();
			if(name && strlen(name))
			{
				const char *tmp=strstr(colstr,name);
				while(tmp)
				{
					if(tmp[-1]==':' && tmp[strlen(name)]==':')
					{
						col->Enable();
						tmp=NULL;
					}
					else
						tmp=strstr(tmp+1,name);
				}
			}
			col=col->NextColorant();
		}
	}
}


// Builds a DeviceNColorantList from an stp_vars_t's
// ChannelNames variable.
// Dummy entries will be created for unknown channels,
// to retain the 1:1 mapping between colorant index and
// printer raw channels.

static const char *ChannelOrder[]=
{
	"Black","Matte Black","Photo Black","Light Black","Light Light Black","Cyan","Light Cyan",
	"Magenta","Light Magenta","Yellow","Red","Blue","Gloss Optimizer",NULL
};


void GPColorantList::BuildList(stp_vars_t *vars)
{
	const char *mode=stp_get_string_parameter(vars,"InputImageType");
	if(strcmp(mode,"Raw")==0)
	{
		BuildList_Raw(vars);
	}
	else if(strcmp(mode,"RGB")==0)
	{
		new GPColorant(*this,"Red");
		new GPColorant(*this,"Green");
		new GPColorant(*this,"Blue");
	}
	else if(strcmp(mode,"CMYK")==0)
	{
		new GPColorant(*this,"Cyan");
		new GPColorant(*this,"Magenta");
		new GPColorant(*this,"Yellow");
		new GPColorant(*this,"Black");
	}
}


void GPColorantList::BuildList_Raw(stp_vars_t *vars)
{
	stp_parameter_t desc;
	stp_describe_parameter(vars,"ChannelNames",&desc);
	stp_string_list_t *strlist=desc.bounds.str;
	if(strlist)
	{
		int i=0;
		int ncol=stp_string_list_count(strlist);
		cerr << "Have " << ncol << " raw channels" << endl;
		while(ChannelOrder[i])
		{
			for(int j=0;j<ncol;++j)
			{
				stp_param_string_t *p=stp_string_list_param(strlist,j);
				cerr << "Comparing " << p->name << " against " << ChannelOrder[i] << endl;
				if(strcmp(p->name,ChannelOrder[i])==0)
				{
					try
					{
						cerr << "-> Adding " << p->name << endl;
						new GPColorant(*this,p->name);
					}
					catch(const char *err)
					{
						cerr << "Ignoring colorant: " << p->name << endl;
						new GPColorant(*this);	// We include a dummy colorant for unrecognised
												// channels (gloss optimiser, mainly) so that
												// each colorant's position in the list continues
												// to match its raw channel number.
					}
					j=ncol;
				}
			}
			++i;
		}
	}
	stp_parameter_description_destroy(&desc);
}


#if 0
void GPColorantList::BuildList(stp_vars_t *vars)
{
	stp_parameter_t desc;
	stp_describe_parameter(vars,"ChannelNames",&desc);
	stp_string_list_t *strlist=desc.bounds.str;
	if(strlist)
	{
		int ncol=stp_string_list_count(strlist);
		cerr << "Have " << ncol << " raw channels" << endl;
		for(int j=0;j<ncol;++j)
		{
			stp_param_string_t *p=stp_string_list_param(strlist,j);
			cerr << "Ink: " << p->name << endl;
			try	
			{
				new GPColorant(*this,p->name);
			}
			catch(const char *err)
			{
				cerr << "Ignoring colorant: " << p->name << endl;
				new GPColorant(*this);	// We include a dummy colorant for unrecognised
										// channels (gloss optimiser, mainly) so that
										// each colorant's position in the list continues
										// to match its raw channel number.
			}
		}
	}
	stp_parameter_description_destroy(&desc);
}
#endif


/////////////////////////////////////////////////////////////////////////////


GPColorant::GPColorant(DeviceNColorantList &header,const char *name) : DeviceNColorant(header,name), enabled(true)
{
}


// Dummy entry
GPColorant::GPColorant(DeviceNColorantList &header) : DeviceNColorant(header,NULL,255,255,255), enabled(false)
{
}


GPColorant::~GPColorant()
{
}


void GPColorant::Enable()
{
	enabled=true;
}


void GPColorant::Disable()
{
	enabled=false;
}


bool GPColorant::GetEnabled()
{
	return(enabled);
}

GPColorant *GPColorant::NextColorant()
{
	return((GPColorant *)DeviceNColorant::NextColorant());
}

GPColorant *GPColorant::PrevColorant()
{
	return((GPColorant *)DeviceNColorant::PrevColorant());
}

