#include <iostream>
#include <stdlib.h>
#include <string.h>
#include <libgen.h>

#include "support/searchpath.h"
#include "support/util.h"

#include "imagesource/imagesource_devicen_remap.h"

#include "gpdevicensupport.h"
#include "linproject.h"

using namespace std;


ConfigTemplate LinProject::Template[]=
{
	ConfigTemplate("ChartSeed",int(0)),	// Random number used when generating charts.
										// Storing this number makes chart generation repeatable.
	ConfigTemplate("Instrument","DTP41"),
	ConfigTemplate("StepsPerChannel",int(64)),
	ConfigTemplate("Colorants",""),
	ConfigTemplate()
};


LinProject::LinProject(LinearizationCore &core,const char *fn)
	: ConfigFile(), ConfigDB(Template), printoutput(this,"[Output]"), gprinter(printoutput,this,"[Print]"),
	core(core), filename(NULL), tempdirname(NULL), temproot(NULL)
{
	new ConfigDBHandler(this,"[GPLin]",this);
	SetInt("ChartSeed",rand()%100);
	SetString("Instrument",core.FindString("DefaultInstrument"));
	if(fn)
	{
		SetFilename(fn);
		Load();
	}
}


void LinProject::SetFilename(const char *fn)
{
	if(filename)
		free(filename);
	if(tempdirname)
		free(tempdirname);
	if(temproot)
		free(temproot);
	filename=strdup(fn);
	tempdirname=BuildFilename(filename,"_temp",NULL);
	CreateDirIfNeeded(tempdirname);

	char *tmp=BuildFilename(filename,"",NULL);
	char *base=basename(tmp);
	temproot=(char *)malloc(strlen(base)+strlen(tempdirname)+4);
	sprintf(temproot,"%s%c%s",tempdirname,SEARCHPATH_SEPARATOR,base);
	free(tmp);
}


const char *LinProject::GetFilename()
{
	return(filename);
}


const char *LinProject::GetTempDir()
{
	return(tempdirname);
}


char *LinProject::GetTempRoot(const char *id,const char *ext)
{
	if(!ext)
		ext="";
	if(!temproot)
		return(NULL);
	int n=strlen(temproot)+strlen(id)+strlen(ext)+2;
	char *result=(char *)malloc(n);
	sprintf(result,"%s_%s%s",temproot,id,ext);
	return(result);
}


LinProject::~LinProject()
{
	if(filename)
		free(filename);
	if(tempdirname)
		free(tempdirname);
	if(temproot)
		free(temproot);
}


bool LinProject::Load()
{
	bool result=false;
	if(filename)
	{
		result=ParseConfigFile(filename);
		printoutput.DBToQueues();
	}
	return(result);
}


bool LinProject::Save()
{
	printoutput.QueuesToDB();
	if(filename)
		return(SaveConfigFile(filename));
	return(false);
}


bool LinProject::Import(const char *file)
{
	bool result=false;
	if(file)
	{
		result=ParseConfigFile(file);
		printoutput.DBToQueues();
	}
	return(result);
}


void LinProject::CreateRawChannelLinChart(const char *id,Progress *p)
{
	cerr << "Generating test target..." << endl;
	GPColorantList colorants(&gprinter);
	colorants.SetEnabledColorants(FindString("Colorants"));

	char *fnroot=GetTempRoot(id);
	TI1File *ti1=core.CreateRawChannelLinChart(fnroot,&colorants,FindInt("StepsPerChannel"),p);
	ArgyllInstrument inst(FindString("Instrument"));

	gprinter.GetImageableArea();
	float pw=gprinter.pagewidth;
	float ph=gprinter.pageheight;
	cerr << "Using pagesize: " << pw << ", " << ph << endl;
	pw=(pw*25.4)/72;
	ph=(ph*25.4)/72;
	ArgyllPaperSize ps(int(pw+0.5),int(ph+0.5));
	int res=core.FindInt("ChartResolution");
	Argyll_TestChart_TIFFs *tiffs=core.RenderTIFFs(fnroot,ps,inst,res,FindInt("ChartSeed"),p);
	delete tiffs;
	delete ti1;
	free(fnroot);
}


TI1File *LinProject::GetRawChannelLinChart(const char *id)
{
	char *fnroot=GetTempRoot(id,".ti1");
	TI1File *ti1=new TI1File(fnroot);
	free(fnroot);
	return(ti1);
}


Argyll_TestChart_TIFFs *LinProject::GetRawChannelLinTIFFs(const char *id)
{
	char *fnroot=GetTempRoot(id);
	Argyll_TestChart_TIFFs *tiffs=new Argyll_TestChart_TIFFs(fnroot);
	free(fnroot);
	return(tiffs);
}


int *LinProject::CreateRawChannelMap(DeviceNColorantList &colorants,TI1File &ti1)
{
	int colcount=colorants.GetColorantCount();
	int *channelmap=new int[colcount];
	for(int i=0;i<colcount;++i)
		channelmap[i]=0;

	DeviceNColorant *col=colorants.FirstColorant();
	int i=0;
	while(col)
	{
		try
		{
			int idx=ti1.GetColorantIndex(col->GetName());
			if(idx>-1)
				channelmap[idx]=i;
			else
				cerr << "-1: Channel not found" << endl;
		}
		catch(const char *err)
		{
			cerr << "Warning: " << err << endl;
		}
		col=col->NextColorant();
		++i;
	}
	for(i=0;i<colcount;++i)
		cerr << "Mapping " << i << " to " << channelmap[i] << endl;
	return(channelmap);
}


const char *LinProject::GetChannelMode()
{
	return(stp_get_string_parameter(gprinter.stpvars,"InputImageType"));
}


void LinProject::SetChannelMode(const char *mode)
{
	stp_set_string_parameter(gprinter.stpvars,"InputImageType",mode);
}


void LinProject::Print(ImageSource *is,int xpos,int ypos,bool bypasslinearization)
{
	const char *mode=GetChannelMode();
	if(strcmp(mode,"RGB")==0)
	{
		is=new ImageSource_DeviceNToRGB(is);
	}
	else if(strcmp(mode,"CMYK")==0)
	{
		is=new ImageSource_DeviceNToCMYK(is);
	}
	if(bypasslinearization)
		gprinter.GPrinter::Print(is,0,0);
	else
		gprinter.Print(is,0,0);
}

