#include <iostream>
#include <string.h>

#include "config.h"
#include "support/thread.h"
#include "argyll_support.h"
#include "ticgats.h"

using namespace std;


CGATSFile::CGATSFile(const char *filename,const char *type) : cgatsfile(NULL)
{
	cgatsfile=new_cgats();
	if(type)
		cgatsfile->add_other(cgatsfile,type);
	if (cgatsfile->read_name(cgatsfile,filename))
		throw cgatsfile->err;
}


CGATSFile::~CGATSFile()
{
	if(cgatsfile)
		cgatsfile->del(cgatsfile);
}


const char *CGATSFile::GetKeyword(int table,const char *key)
{
	int i=cgatsfile->find_kword(cgatsfile,table,key);
	if(i>=0)
	{
		return(cgatsfile->t[table].kdata[i]);
	}
	return(NULL);
}


const char *CGATSFile::GetFieldName(int table,int i)
{
	return(cgatsfile->t[table].fsym[i]);
}


int CGATSFile::FindField(int table,const char *field)
{
	return(cgatsfile->find_field(cgatsfile,table,field));
}


////////////////////////////////////////////////////////////////////////////////////////////////////

// TI1File handles the small amount of data we need from a TI1 file - namely the colorants and
// the order in which they appear.


TI1File::TI1File(const char *filename)
	: CGATSFile(filename,"CTI1"), DeviceNColorantList()
{
	colorantnames=GetKeyword(0,"COLOR_REP");

	const char *cn=colorantnames;
	while(*cn)
	{
		const char *longname=ArgyllSupport::GetColorantLongName(cn);
		if(longname)
			new DeviceNColorant(*this,longname);
		if(cn[0]=='2' || cn[0]=='3')
			++cn;
		++cn;
	}
}

TI1File::~TI1File()
{
}


static struct { const char *name; const char *alias;} fuzzymatches[]=
{
	{"Photo Black","Black"},
	{"Matte Black","Medium Black"},
	{NULL,NULL}
};

int TI1File::GetColorantIndex(const char *name)
{
	if(!name)
		return(-1);
	int result=DeviceNColorantList::GetColorantIndex(name);
	cerr << "Matching " << name << endl;
	if(result<0)
	{
		int idx=0;
		while(fuzzymatches[idx].name)
		{
			if(strcmp(name,fuzzymatches[idx].name)==0)
			{
				result=DeviceNColorantList::GetColorantIndex(fuzzymatches[idx].alias);
				if(result>-1)
					return(result);
			}
			++idx;
		}
		throw "Colorant not found";
	}
	return(result);
}


////////////////////////////////////////////////////////////////////////////////////////////////////

// TI3File handles specific aspects of Argyll's TI3 format.  Notably, field names and indices are
// offset by one, to ignore the SAMPLE_ID field.


TI3File::TI3File(const char *filename)
	: CGATSFile(filename,"CTI3")
{
	table=cgatsfile->t;
	sets=table->nsets;

	int nfields=table->nfields-1;
	for(int i=0;i<nfields;++i)
	{
		const char *fn=GetFieldName(i);
		if(strncmp(fn,"XYZ",3)==0)
		{
			colorants=i;
			i=nfields;
		}
	}	
	Xindex=FindField("XYZ_X");
	Yindex=FindField("XYZ_Y");
	Zindex=FindField("XYZ_Z");

	cerr << colorants << " colorants and " << sets << " measurements" << endl;
}

TI3File::~TI3File()
{
}

int TI3File::GetColorantCount()
{
	return(colorants);
}

int TI3File::GetSets()
{
	return(sets);
}

const char *TI3File::GetFieldName(int i)
{
	return(CGATSFile::GetFieldName(0,i+1));
}

int TI3File::FindField(const char *field)
{
	// Offset by 1, to skip SAMPLE_ID
	return(CGATSFile::FindField(0,field)-1);
}

double **TI3File::GetData(int set)
{
	return(((double **)table->fdata[set])+1);
}

